@@ -4,7 +4,7 @@ from .definitions import (
 )
 from .input_sequences import (
     YES_INDICATING_STRINGS, NO_INDICATING_STRINGS, EDIT_INDICATING_STRINGS,
-    CHAIN_SKIP_INDICATING_STRINGS, DOC_SKIP_INDICATING_STRINGS,
+    CHAIN_SKIP_INDICATING_STRINGS, DOC_SKIP_INDICATING_STRINGS
 )
 from typing import Optional, BinaryIO, Union, cast, Iterator
 import os
@@ -16,7 +16,9 @@ from selenium.common.exceptions import WebDriverException as SeleniumWebDriverEx
 from selenium.webdriver.remote.webdriver import WebDriver as SeleniumWebDriver
 import binascii
 import requests
-from . import progress_report, content_match, scr, utils, scr_context
+from . import (
+    progress_report, content_match, scr, utils, scr_context, selenium_setup
+)
 from collections import OrderedDict
 import threading
 import concurrent.futures
@@ -384,7 +386,7 @@ class DownloadJob:
                 req = scr.request_raw(
                     self.cm.mc.ctx, self.cm.clm.result, cast(
                         urllib.parse.ParseResult, self.cm.url_parsed),
-                    scr.load_selenium_cookies(self.cm.mc.ctx),
+                    selenium_setup.load_selenium_cookies(self.cm.mc.ctx),
                     proxies=proxies, stream=True
                 )
                 self.content = ResponseStreamWrapper(req)
@@ -406,7 +408,7 @@ class DownloadJob:
             return False
 
     def selenium_download_internal(self) -> bool:
-        doc_url_str = scr.selenium_get_url(self.cm.mc.ctx)
+        doc_url_str = selenium_setup.selenium_get_url(self.cm.mc.ctx)
         if doc_url_str is None:
             return False
         doc_url = urllib.parse.urlparse(doc_url_str)
@@ -431,11 +433,11 @@ class DownloadJob:
             document.body.removeChild(a);
         """
         try:
-            scr.selenium_exec_script(self.cm.mc.ctx, script_source,
-                                     self.cm.clm.result, tmp_filename)
+            selenium_setup.selenium_exec_script(self.cm.mc.ctx, script_source,
+                                                self.cm.clm.result, tmp_filename)
         except SeleniumWebDriverException as ex:
-            if scr.selenium_has_died(self.cm.mc.ctx):
-                scr.report_selenium_died(self.cm.mc.ctx)
+            if selenium_setup.selenium_has_died(self.cm.mc.ctx):
+                selenium_setup.report_selenium_died(self.cm.mc.ctx)
             else:
                 scr.log(
                     self.cm.mc.ctx, Verbosity.ERROR,
@@ -454,7 +456,7 @@ class DownloadJob:
                 time.sleep(0.1)
                 if i > 15:
                     i = 10
-                    if scr.selenium_has_died(self.cm.mc.ctx):
+                    if selenium_setup.selenium_has_died(self.cm.mc.ctx):
                         return False
 
             i += 1
@@ -499,11 +501,11 @@ class DownloadJob:
         driver = cast(SeleniumWebDriver, self.cm.mc.ctx.selenium_driver)
         try:
             doc_url = driver.current_url
-            res = scr.selenium_exec_script(
+            res = selenium_setup.selenium_exec_script(
                 self.cm.mc.ctx, script_source, self.cm.clm.result)
         except SeleniumWebDriverException as ex:
-            if scr.selenium_has_died(self.cm.mc.ctx):
-                scr.report_selenium_died(self.cm.mc.ctx)
+            if selenium_setup.selenium_has_died(self.cm.mc.ctx):
+                selenium_setup.report_selenium_died(self.cm.mc.ctx)
                 return False
             err = str(ex)
         if "error" in res:
@@ -835,31 +837,25 @@ class DownloadManager:
                 x.result()
             self.pending_jobs.clear()
         self.pom.request_print_access()
-        committed_report_line_count = 0
-        report_lines: list[progress_report.StatusReportLine] = []
+        prm = progress_report.ProgressReportManager()
         while True:
             results = concurrent.futures.wait(
                 self.pending_jobs,
-                timeout=0 if not committed_report_line_count
+                timeout=0 if not prm.prev_report_line_count
                 else progress_report.DOWNLOAD_STATUS_REFRESH_INTERVAL
             )
             for x in results.done:
                 x.result()
             self.pending_jobs = results.not_done
-            with self.status_report_lock:
-                progress_report.load_status_report_lines(
-                    self.download_status_reports, report_lines
-                )
-            if len(report_lines) == 0:
+            prm.load_status(self)
+            if not prm.updates_remaining():
                 if not self.pending_jobs:
                     # this happens when we got main thread print access
                     # but everybody is already done and never downloaded anything
                     # we don't want any progress reports here
                     break
                 continue
-            committed_report_line_count = progress_report.print_status_report(
-                report_lines, committed_report_line_count
-            )
+            prm.print_status_report()
             if not self.pending_jobs:
                 break
 
