@@ -1,20 +1,24 @@
-from typing import Optional
-import os
+from . import windows
+from typing import Optional, Callable
+import platform
 import sys
 import re
+import select
+from .definitions import T
 
 
 def is_windows() -> bool:
-    return os.name == "nt"
+    return platform.system() == 'Windows'
+
+
+def is_osx() -> bool:
+    return platform.system() == 'Darwin'
+
+
+def is_linux() -> bool:
+    return platform.system() == 'Linux'
 
 
-if is_windows():
-    import win32api
-    import win32event
-    import win32file
-    import pywintypes
-else:
-    import select
 DEFAULT_TRUNCATION_LENGTH = 200
 
 
@@ -39,24 +43,23 @@ def empty_string_to_none(string: Optional[str]) -> Optional[str]:
     return string
 
 
+def choose_first_not_none(*tries: Callable[[], Optional[T]]) -> Optional[T]:
+    for t in tries:
+        res = t()
+        if res is not None:
+            return res
+    return None
+
+
 def stdin_has_content(timeout: float) -> bool:
     assert timeout >= 0
-    if not is_windows():
+    if sys.platform != 'win32':
         rlist, _, _ = select.select(
             [sys.stdin], [], [], timeout
         )
         return bool(rlist)
     else:
-        try:
-            # without this the wait sometimes returns without there being
-            # any actual data -> we woul block infinitely on the read
-            win32file.FlushFileBuffers(win32api.STD_INPUT_HANDLE)
-        except pywintypes.error:
-            # the flush sometimes fails, too bad!
-            pass
-        return win32event.WaitForSingleObject(
-            win32api.STD_INPUT_HANDLE, int(timeout * 1000)  # milliseconds
-        ) is win32event.WAIT_OBJECT_0
+        return windows.stdin_has_content(timeout)
 
 
 def remove_file_scheme_from_url(url: str) -> str:
